/* jmixer.h
 * (c)2000 Denis Roio aka jaromil
 * based on the eMixer code by Dunstan B. Nesbit
 */

#ifndef __JMIXER_H
#define __JMIXER_H

#include <unistd.h>
#include <stdlib.h>

extern "C" {
#include "shout.h"
#include "lame/lame.h"
#include "mpeg.h"
}

#define BUF_SIZE 2048
#define MAX_SAMPLE 32766
#define CLIP 30000
#define CLIP_A ((MAX_SAMPLE-CLIP)*(MAX_SAMPLE-CLIP)) // 7650756
#define CLIP_B (MAX_SAMPLE-2*CLIP) // 982980000
#define MAX_CHANNELS 8
#define MAX_PLAYLIST_ENTRY 100
#define MAX_PATH_SIZE 240

struct channel {
  short inbuf[BUF_SIZE];
  char *name;
  float volume;
  float speed;
  int rate;
  int channels;
  bool on;
  int end;
  int stayon;
  int stream;
  FILE *fstr;
  pid_t pid;
  int chindx;
};

class Stream_mixer {

private:

  char playlist[MAX_PLAYLIST_ENTRY][MAX_PATH_SIZE];

  short audio_buffer[BUF_SIZE];
  int process_buffer[BUF_SIZE];
  short work1[BUF_SIZE*40], work2[BUF_SIZE];
  short inbuf[BUF_SIZE*40];

  //  int enc_in;
  //  int enc_out;
  int dsp;
  shout_conn_t ice_conn;
  lame_global_flags enc_flags;
  enum { DSP, ICECAST } output;
  
  /* conf & m_dat */
  int sample_rate;
  int stereo;
  int c_number;
  int rate;
  int channels;
  channel stream_channel[MAX_CHANNELS];
  
  bool write_toicecast(short *sendbuf, long len);
  void close_soundcard();

  void get_audio(int ch);
  int spawn_stream(int ch);
  int spawn_encoder();
  void kill_channel(int ch);
  int init_pipe(char *file);

  void make_mono(short *stereo, short *mono, int num);
  void make_stereo(short *mono, short *stereo, int num);
  void sep_stereo(short *stereo, short *left, short *right, int num);
  void mix_stereo(short *left, short *right, short *stereo, int num);

  void int_compress_samples(short *in, short *out, int num, int grain);
  void int_stretch_samples(short *in, short *out, int num, int grain);

  void compress_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void change_time(short *in, short *out, unsigned int num_in, unsigned int num_out);
  void convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2);
  
public:
  Stream_mixer(char *pl);
  
  bool open_soundcard();
  bool open_icecast(char *ip, int port, char *pass);
  unsigned int play_chunks();
  void start_channel(int ch);
  void stop_channel(int ch);
  void set_track(int ch, int playlist_pos);
  void set_volume(int ch, float vol);
  void set_speed(int ch, float speed);

  void set_playlist(char *pl);
  void print_playlist();

  ~Stream_mixer();
};

#endif
